import BigNumber from 'bignumber.js';
import { AbiParamsCommon } from '../types/ABI.js';
export declare function isValidURL(url: string): boolean;
export declare function isObject(obj: unknown): obj is Record<number | string | symbol, unknown>;
export declare function isArray(array: unknown): array is unknown[];
export declare function isJson(string: string): boolean;
export declare function isBoolean(bool: unknown): bool is boolean;
export declare function isBigNumber(number: unknown): number is BigNumber;
export declare function isString(string: unknown): string is string;
export declare function isFunction(obj: unknown): obj is (...args: unknown[]) => unknown;
export declare function isHex(string: string): string is string;
export declare function isInteger(number: unknown): number is number;
export declare function hasProperty(obj: object, property: string | number | symbol): boolean;
export declare function hasProperties(obj: object, ...properties: (string | number | symbol)[]): boolean | 0;
export type EventQueryDataType = {
    block_number: number;
    block_timestamp: number;
    contract_address: string;
    event_name: string;
    transaction_id: string;
    result: unknown;
    resource_Node: string;
    _unconfirmed?: boolean;
    _fingerprint?: string | undefined;
};
export type MapEventQueryDataType = {
    block: number;
    timestamp: number;
    contract: string;
    name: string;
    transaction: string;
    result: unknown;
    resourceNode: string;
    unconfirmed?: boolean;
    fingerprint?: string | undefined;
};
export declare function mapEvent(event: EventQueryDataType): MapEventQueryDataType;
export declare function parseEvent(event: EventQueryDataType, { inputs: abi }: {
    inputs: ReadonlyArray<AbiParamsCommon>;
}): EventQueryDataType;
export declare function padLeft(input: string | number, padding: string, amount: number): string;
export declare function isNotNullOrUndefined(val: unknown): boolean;
export declare function sleep(millis?: number): Promise<any>;
